from collections import defaultdict

import pandas as pd
import yaml
from pandas import DataFrame
from tabulate import tabulate
import wandb
from evaluation.eval_settings import EvaluationSettings, TabulateConfig


def to_pandas(eval_configs):
    data = {}
    for config in eval_configs:
        data[config['id']] = {**config['results'], **config['resolved_vars']}

    return pd.DataFrame.from_dict(data, orient='index')




# def get_table(eval_configs, view):


def by_config_mapping(configs):
    full_results = list(map(lambda x: EvaluationSettings(**x), configs))
    mapping = defaultdict(list)
    for cfg in full_results:
        mapping[cfg.name].append(cfg)
    return mapping


def print_tables(configs):
    mapping = by_config_mapping(configs)
    # full_results = list(map(lambda x: EvaluationConfig(**x), configs))
    #
    # mapping = defaultdict(list)
    # for cfg in full_results:
    #     mapping[cfg.name].append(cfg)
    for name in mapping:
        table = get_table(mapping[name])
        table_length = table.find('\n')
        print('-' * table_length)
        # middle =
        if name:
            print('|' + name.rjust((table_length + 1) // 2), ' ' * (table_length // 2 - 3) + '|', )
            print('-' * table_length)
        print(table)
        print('-' * table_length)
        print()


def log_algo_curve(df: DataFrame, x_keys, project_name, plot_name=None, ):
    for algo in df['algo'].unique():
        wandb.init(name=algo, project='pogema-eval-' + project_name, anonymous="allow")
        # for map_name in df['map_name'].unique():

        m = df.loc[(df['algo'] == algo)]
        for metric in df.head():
            if metric in x_keys or metric in ['algo', 'map_name']:
                continue

            for x in x_keys:
                table = wandb.Table(data=list(zip(m[x], m[metric])), columns=[x, metric])
                wandb.log({metric + x: wandb.plot.line(table, x, metric, title=f"{plot_name}")})

        wandb.finish()


def log_plots(configs, project_name):
    mapping = by_config_mapping(configs)
    for config_name, configs in mapping.items():
        cfg = configs[0]

        x_keys = []
        for key in cfg.resolved_vars:
            if key == 'algo' or key in cfg.tabulate_config.drop_keys:
                continue
            x_keys.append(key)
        if len(x_keys) != 1:
            print('skipping graph drawing with multiple x keys')
            continue
        df = join_by_seed(mapping[config_name])
        log_algo_curve(df, x_keys=x_keys, plot_name=config_name, project_name=project_name)


def print_from_file(path):
    from yaml import CLoader as Loader
    with open(path, "r") as f:

        configs = yaml.load(f, Loader=Loader)
        # configs = yaml.safe_load(f)
    print_tables(configs)


def main():
    with open("../results/APPO.yaml", 'r') as handle:
        for event in yaml.parse(handle):
            print(event)
    # print_from_file("../results/APPO.yaml")


if __name__ == '__main__':
    main()
