from planning.decentralized import AStarHolder, AlgoDecentralized
from evaluation.eval_settings import AlgoAPPO
from evaluation.policies import APPOHolder
from stochastic_grid.config import StochasticGridConfig
import gym

import os


def run(use='ppo'):
    os.environ['OMP_NUM_THREADS'] = "1"
    os.environ['MKL_NUM_THREADS'] = "1"
    if use == 'ppo':
        algo = APPOHolder(algo_cfg=AlgoAPPO(path_to_weights='results/so', ), )
    elif use == 'a_star':
        algo = AStarHolder(cfg=AlgoDecentralized(device='cpu'))
    else:
        raise KeyError(f'No algo with name: {use}')

    for map_name in ['wc3-tranquilpaths', 'street-Sydney_2', 'sc1-WheelofWar']:
        sgc = StochasticGridConfig(seed=0, obs_radius=5, max_episode_steps=512,
                                   num_obstacles=16, map_name=map_name,
                                   size_range=[5, 10], shake_r=5,
                                   show_range=[8, 16], hide_range=[8, 16], so_density=0.7, num_agents=1)

        env = gym.make('StochasticGrid-v0', grid_config=sgc, with_animations=True, auto_reset=False, egocentric_idx=0)

        dones = [False, ...]
        obs = env.reset()
        algo.after_reset(env)

        while not all(dones):
            action = algo.act(obs)
            obs, _, dones, info = env.step(action)
            algo.after_step(dones)
        env.save_animation(f'animation-{use}-{map_name}.svg')


def main():
    run(use='ppo')
    run(use='a_star')


if __name__ == '__main__':
    main()
