import pathlib
import wandb


def load_artifact(name, type_, load_to=None):
    if load_to is None:
        load_to = f'results/{name}({type_})'
    if pathlib.Path(load_to).exists():
        print(f'Skipping already exist weights: {name} at {pathlib.Path(load_to)}')
        return load_to

    wb_run = wandb.init()
    artifact = wb_run.use_artifact(name, type=type_)

    artifact_dir = artifact.download(root=load_to)
    wandb.finish()
    return artifact_dir


def load_inference_weights():
    name = 'tviskaron/StochasticObstacles/so:v0'
    load_artifact(name, type_='ppo', load_to=f'results/so')


def upload_weights(path, name='so', type_='ppo'):
    wb_run = wandb.init(project='StochasticObstacles')
    wb_run.log_artifact(path, name=name, type=type_)


def main():
    # upload_weights('results/so')
    load_inference_weights()


if __name__ == '__main__':
    main()
